import datetime

from sqlalchemy import DateTime
from sqlalchemy.ext.mutable import Mutable
from sqlalchemy.orm import Mapped, declarative_mixin, mapped_column
from sqlalchemy.sql import functions

# from sqlalchemy.types import BINARY, DECIMAL, JSON, BigInteger


# from sqlalchemy.ext.compiler import compiles


class MutableList(Mutable, list):
    """This allow modify an ARRAY custom type
    From https://gist.github.com/kirang89/22d111737af0fca251e3

    """

    def __setitem__(self, key, value):
        list.__setitem__(self, key, value)
        self.changed()

    def __delitem__(self, key):
        list.__delitem__(self, key)
        self.changed()

    def append(self, value):
        list.append(self, value)
        self.changed()

    def pop(self, index=0):
        value = list.pop(self, index)
        self.changed()
        return value

    @classmethod
    def coerce(cls, key, value):
        if not isinstance(value, MutableList):
            if isinstance(value, list):
                return MutableList(value)
            return Mutable.coerce(key, value)
        else:
            return value


@declarative_mixin
class DateTrackMixin:
    created_at: Mapped[datetime.datetime] = mapped_column(
        DateTime(),
        server_default=functions.now(),
        nullable=False,
    )
    updated_at: Mapped[datetime.datetime] = mapped_column(
        DateTime(),
        server_default=functions.now(),
        nullable=False,
    )



import click
from rich.console import Console
from rich.prompt import Prompt
from rich.table import Table

from services import conf, defaults, errors, types, utils
from services.security import jwtauth
from services.security.memory_store import MemoryTokenStore

console = Console()


@click.group(name="tokens")
def tokens_cli():
    """
    Token generator
    """
    pass


@tokens_cli.command(name="generate")
@click.option(
    "--settings-module",
    "-s",
    default=defaults.SETTINGS_MODULE,
    help="Fullpath to settings module",
)
@click.option("--scopes", "-s", default="user:r:w", help="Scopes for the token")
@click.option("--exp", "-e", default=600, help="Expiration of the token")
@click.option("--username", "-u", default=None, help="Username")
def generate(settings_module, username, scopes, exp):
    settings = conf.load_conf(settings_module)
    custom = {"scopes": scopes}
    if username:
        custom.update({"usr": username})
    payload = types.JWTPayload(custom=custom, exp=exp)

    jwt = jwtauth.JWTAuth(settings.SECURITY)
    token = utils.from_sync2async(jwt.generate_token, payload)

    console.print(token.dict())


@tokens_cli.command(name="decode")
@click.option(
    "--settings-module",
    "-s",
    default=defaults.SETTINGS_MODULE,
    help="Fullpath to settings module",
)
@click.argument("token")
def decode(settings_module, token):
    settings = conf.load_conf(settings_module)
    jwt = jwtauth.JWTAuth(settings.SECURITY)
    decoded = jwt.decode(token)

    console.print(decoded)
